"""
Contains shared variables and functions for the web app.
"""

# Default tags that must be present on all resources.
WEBAPP_TAGS = {
    # The resource group is created by weblodge
    # and contains a WebApp managed by it.
    'contains': 'webapp'
}
